import React, { useState, useEffect } from 'react'
import fetch from 'unfetch'
import Head from 'next/head'
import Link from 'next/link'

import LoadingIndicator from '../components/LoadingIndicator'
import Sleep from '../components/sleep'
import NewsEntry from '../components/newsEntry'
import DbSelector from '../components/dbSelector.js'

import styles from './index.module.css'
import constants from '../constants.json'

const links = {
  install: {
    href: '/install',
    text: 'Install',
  },
  irc: {
    href: '/irc',
    text: 'IRC chat',
  },
}

function fetchNews() {
  return new Promise((resolve) => {
    fetch(`${constants.apiUrl}news.json`, {
      credentials: 'include',
    })
      .then((resp) => {
        return resp.json()
      })
      .then((res) => {
        resolve(res)
      })
  })
}

export default function Index({ globalData }) {
  const userId = globalData.userId
  const motd = globalData.motd
  const [news, setNews] = useState(null)
  const [allQrSelector, setAllQrSelector] = useState(null)
  // const userSpecificMotd = props.globalData.userSpecificMotd

  useEffect(() => {
    console.info('Fetching news.json')
    fetchNews().then((res) => {
      setNews(res)
    })
  }, [])

  const renderNews = () => {
    if (news) {
      let newsItems = Object.keys(news)
        .map((key) => {
          let newsEntryData = news[key]
          return (
            <NewsEntry
              onReact={({ type, path, reaction, isDelete }) => {
                if (type === 'news') {
                  fetch(constants.apiUrl + 'infos', {
                    method: 'POST',
                    credentials: 'include',
                    headers: {
                      Accept: 'application/json',
                      'Content-Type': 'application/json',
                    },
                    body: JSON.stringify({
                      react: reaction,
                      newsKey: key,
                      isDelete: isDelete,
                    }),
                  }).then((res) => {
                    // TODO: dont refetch news
                    fetchNews().then((res) => {
                      setNews(res)
                    })
                  })
                } else if (type === 'comment') {
                  fetch(constants.apiUrl + 'comment', {
                    method: 'POST',
                    credentials: 'include',
                    headers: {
                      Accept: 'application/json',
                      'Content-Type': 'application/json',
                    },
                    body: JSON.stringify({
                      type: 'reaction',
                      newsKey: key,
                      path: path,
                      reaction: reaction,
                      isDelete: isDelete,
                    }),
                  }).then((res) => {
                    fetchNews().then((res) => {
                      setNews(res)
                    })
                  })
                }
              }}
              onDelete={(path) => {
                fetch(constants.apiUrl + 'comment', {
                  method: 'POST',
                  credentials: 'include',
                  headers: {
                    Accept: 'application/json',
                    'Content-Type': 'application/json',
                  },
                  body: JSON.stringify({
                    type: 'delete',
                    path: path,
                    newsKey: key,
                  }),
                }).then(() => {
                  fetchNews().then((res) => {
                    setNews(res)
                  })
                })
              }}
              onComment={(path, text) => {
                fetch(constants.apiUrl + 'comment', {
                  method: 'POST',
                  credentials: 'include',
                  headers: {
                    Accept: 'application/json',
                    'Content-Type': 'application/json',
                  },
                  body: JSON.stringify({
                    type: 'add',
                    path: path,
                    text: text,
                    newsKey: key,
                  }),
                }).then(() => {
                  fetchNews().then((res) => {
                    setNews(res)
                  })
                })
              }}
              uid={userId}
              key={key}
              newsKey={key}
              newsItem={newsEntryData}
            />
          )
        })
        .reverse()

      return (
        <div>
          <div className={styles.title}>News</div>
          <hr />
          <hr />
          <div>{newsItems}</div>
        </div>
      )
    } else {
      return <LoadingIndicator />
    }
  }

  const renderMotd = () => {
    return (
      <div>
        <div className={styles.title}>MOTD</div>
        <hr />
        <hr />
        {motd ? (
          <div
            className={styles.motd}
            dangerouslySetInnerHTML={{ __html: motd }}
          />
        ) : (
          <div>...</div>
        )}
      </div>
    )
  }

  // const renderUserSpecificMotd = () => {
  //   return (
  //     <div>
  //       <hr />
  //       <div className={styles.subtitle}>Üzenet admintól:</div>
  //       {userSpecificMotd ? (
  //         <div
  //           className={styles.motd}
  //           dangerouslySetInnerHTML={{ __html: userSpecificMotd.msg }}
  //         />
  //       ) : (
  //         <div>...</div>
  //       )}
  //     </div>
  //   )
  // }

  const renderDbSelector = () => {
    if (allQrSelector) {
      return (
        <DbSelector
          text={`Válaszd ki melyik adatbázist szeretnéd letölteni (${allQrSelector}):`}
          showAll={allQrSelector === 'txt'}
          closeClick={() => {
            setAllQrSelector(null)
          }}
          onDbSelect={(selectedDb) => {
            if (allQrSelector === 'txt') {
              if (selectedDb === 'all') {
                window.open(`${constants.apiUrl}allqr.txt`, '_blank')
              } else {
                window.open(
                  `${constants.apiUrl}allqr.txt?db=${selectedDb.name}`,
                  '_blank'
                )
              }
            } else if (allQrSelector === 'json') {
              window.open(`${constants.apiUrl}${selectedDb.path}`, '_blank')
            }
          }}
        />
      )
    } else {
      return null
    }
  }

  return (
    <div>
      <Head>
        <title>Qmining | Frylabs.net</title>
      </Head>
      <div className={styles.buttonContainer}>
        {Object.keys(links).map((key) => {
          let link = links[key]
          return (
            <Link key={key} href={link.href}>
              <a className={styles.button} target="_blank">
                {link.text}
              </a>
            </Link>
          )
        })}
        <a
          onClick={() => {
            setAllQrSelector('txt')
          }}
          className={styles.button}
        >
          {'Összes kérdés TXT'}
        </a>
        <a
          onClick={() => {
            setAllQrSelector('json')
          }}
          className={styles.button}
        >
          {'Összes kérdés JSON'}
        </a>
      </div>
      <hr />
      {renderMotd()}
      {/*{userSpecificMotd && renderUserSpecificMotd()} */}
      <hr />
      <hr />
      <Sleep />
      {renderNews()}
      {renderDbSelector()}
    </div>
  )
}
